@@ -28,7 +28,7 @@ class modelGenerator(ExcelImporter):
              'varchar':{'method':'CharFields','columnType':'String'}
             ,'numeric':{'method':'NumericFields','columnType':'Numeric'}
             ,'date':{'method':'DateFields','columnType':'Date'}
-            ,'datetime':{'method':'DateTimeFields','columnType':'DateTime'}
+            ,'timestamp':{'method':'DateTimeFields','columnType':'DateTime'}
             ,'integer':{'method':'IntFields','columnType':'Integer'}
             ,'double':{'method':'FloatFields','columnType':'Float'}
         }
@@ -125,7 +125,7 @@ class modelGenerator(ExcelImporter):
 
         # pythonファイル出力
         with open(os.path.join(output_dir, sheetname + '.py'), 'w', newline='', encoding='utf-8') as f:
-            f.writelines('import LibHanger.Models.fields as fld\n')
+            f.writelines('import LibHanger.Models.modelFields as fld\n')
             f.writelines('from sqlalchemy.ext.declarative import declarative_base\n')
             f.writelines('from sqlalchemy.sql.elements import Null\n')
             f.writelines('\n')
