@@ -5,10 +5,14 @@ import LibHanger.Library.uwLogger as Logger
 from typing import TypeVar, Generic
 from enum import Enum
 from sqlalchemy.exc import SQLAlchemyError
-from sqlalchemy import and_
+from sqlalchemy.sql.elements import BinaryExpression
+from sqlalchemy import and_, or_
+from sqlalchemy import asc,desc
 from LibHanger.Models.fields import fields
 from LibHanger.Library.DataAccess.Base.uwDataAccess import uwDataAccess
 from LibHanger.Library.uwDeclare import uwDeclare as en
+from LibHanger.Models.saWhere import saWhere
+from LibHanger.Models.saOrderBy import saOrderBy
 
 T = TypeVar('T')
 
@@ -41,6 +45,18 @@ class recset(Generic[T]):
         
         deleted = 3
         """ 削除 """
+    
+    class findOption(Enum):
+        
+        """
+        検索モード
+        """
+        
+        cacheOnly = 0
+        """ キャッシュのみ検索 """
+
+        cacheAndDataBase = 1
+        """ キャッシュとDBの両方検索 """
         
     def __init__(self, t, __da:uwDataAccess = None, __where = None) -> None:
         
@@ -88,6 +104,9 @@ class recset(Generic[T]):
         # フィールドクラス
         self.__fields = fields(self.__rows, self.__dfrows)
         
+        # 一時セッションフラグ
+        self.__tempSessionFlg = False
+        
     @property
     def recSetName(self):
         
@@ -97,6 +116,10 @@ class recset(Generic[T]):
         
         return self.modelType.__tablename__
     
+    @property
+    def recordCount(self):
+        return len(self.rows)
+    
     @property
     def session(self):
         
@@ -394,6 +417,49 @@ class recset(Generic[T]):
         # tupleに変換(dictrowのKeyにする)
         return tuple(pKeyValueList)
 
+    def __checkSessionOpen(self, __autoCommit = True, __beginTransaction = False):
+
+        """
+        セッションチェック(Open)
+
+        Parameters
+        ----------
+        __autoCommit : bool
+            AutoCommit
+        __beginTransaction : bool
+            Sessionを開いた時にトランザクションを開始するか
+
+        """
+        
+        # sessionがNoneの場合はautocommit=Onで接続を開く
+        if self.__session is None:
+            self.__da.openSession(__autoCommit)
+            self.__session = self.__da.session
+            self.__da.beginTransaction()
+            self.__tempSessionFlg = True
+        else:
+            # autocommit=Onの場合はtransactionを開始する
+            if self.__da.autoCommit and \
+               self.session.transaction is None and \
+               __beginTransaction:
+                self.__da.beginTransaction()
+    
+    def __checkSessionClose(self):
+        
+        """
+        セッションチェック(Close)
+
+        Parameters
+        ----------
+        None
+        
+        """
+        
+        if not self.__session is None and self.__tempSessionFlg:
+            self.__da.closeSession()
+            self.__session = None
+            self.__tempSessionFlg = False
+
     def setDA(self, __da:uwDataAccess):
 
         """
@@ -505,7 +571,107 @@ class recset(Generic[T]):
         # 戻り値を返す
         return df
     
-    def filter(self, w):
+    def filterExpr(self, w, s):
+        
+        """
+        フィルタ条件適用
+        
+        Parameters
+        ----------
+        w : Any | asWhere
+            抽出条件
+        s : Any | asOrderBy
+            ソート順
+        """
+        
+        # Where条件
+        if not type(w) is saWhere:
+            q = self.__session.query(self.modelType).filter(w)
+        else:
+            q = self.__session.query(self.modelType)
+            
+            # where条件キャスト
+            wx:saWhere = w
+            
+            # AND
+            if len(wx.andList) > 0:
+                q = q.filter(and_(*tuple(wx.andList)))
+            
+            # OR
+            if len(wx.orList) > 0:
+                q = q.filter(or_(*tuple(wx.orList)))
+        
+        # Sort順
+        if s is None:
+            for pKey in self.__primaryKeys:
+                q = q.order_by(asc(getattr(self.modelType, pKey)))
+        elif not type(s) is saOrderBy:
+            q = q.order_by(s)
+        else:
+            
+            # OrderByキャスト
+            sx:saOrderBy = s
+            
+            # OrderBy 組み立て
+            if len(sx.orderBy) > 0:
+                # keyでソート
+                sort_orderBy = sorted(sx.orderBy.items())
+                for sortKey in sort_orderBy:
+                    q = q.order_by(sortKey[1])
+        
+        # 戻り値を返す
+        return q
+    
+    def convertDataFrameExpr(self, df, w):
+        
+        """
+        sqlAlchemyのfilter条件をDataFrameのfilter条件に変換
+        
+        df : DataFrame
+            対象DataFrame
+        w : any | asWhere
+            where条件
+        """
+        
+        if not type(w) is saWhere:
+            return w
+        else:
+            # where条件キャスト
+            wx:saWhere = w
+
+            # 条件格納用リスト初期化
+            andExprList = []
+            orExprList = []
+            
+            # AND
+            if len(wx.andList) > 0:
+                for expr in wx.andList:
+                    andExpr:BinaryExpression = expr
+                    if not type(andExpr.right.value) is list:
+                        andExprList.append(df[andExpr.left.key] == andExpr.right.value)
+                    else:
+                        dfInExpr = None
+                        for rval in andExpr.right.value:
+                            dfInExprTmp = (df[andExpr.left.key] == rval)
+                            dfInExpr = (dfInExpr | dfInExprTmp) if not dfInExpr is None else dfInExprTmp
+                        andExprList.append(dfInExpr)
+            # OR
+            if len(wx.orList) > 0:
+                for expr in wx.orList:
+                    orExpr:BinaryExpression = expr
+                    orExprList.append(df[orExpr.left.key] == orExpr.right.value)
+            
+            # DataFrame用のwhere条件組み立て
+            retExpr = None
+            for expr in andExprList:
+                retExpr = retExpr & expr if not retExpr is None else expr
+            for expr in orExprList:
+                retExpr = retExpr | expr if not retExpr is None else expr
+
+            # 戻り値を返す
+            return retExpr
+        
+    def filter(self, w, s = None):
         
         """
         レコードセットをフィルタする
@@ -513,8 +679,10 @@ class recset(Generic[T]):
 
         Parameters
         ----------
-        w : any
+        w : any | asWhere
             where条件
+        s : Any | asOrderBy
+            ソート順
         """
         
         # 行情報初期化
@@ -524,11 +692,11 @@ class recset(Generic[T]):
         self.__fields = fields(self.__rows, self.__dfrows)
 
         # セッションチェック
-        self.__checkSession(False, False)
+        self.__checkSessionOpen(False, False)
 
         # クエリ実行
         rowIndex = 0
-        q = self.__session.query(self.modelType).filter(w).all()
+        q = self.filterExpr(w, s).all()
         for row in q:
             # 行状態をnoChangedに変更
             row.__rowState = self.rowState.noChanged
@@ -538,9 +706,44 @@ class recset(Generic[T]):
             self.__addRecrowDataFrame(row, rowIndex)
             rowIndex += 1
         
+        # セッションチェック
+        self.__checkSessionClose()
+        
         # カレント行インデックス初期化
         self.__currentRowIndex = -1
 
+    def find(self, w, searchOption = findOption.cacheOnly):
+        
+        """
+        現在保持しているレコードセットに指定した条件に合致するレコードが存在するか返す
+
+        Parameters
+        ----------
+        w : Any
+            抽出条件
+        searchOption : findOption
+            検索方法
+        """
+        
+        # 条件抽出
+        dfw = self.convertDataFrameExpr(self.__dfrows, w)
+        dfCheck = self.__dfrows[dfw]
+        
+        # filter
+        if searchOption == self.findOption.cacheOnly:
+            # 戻り値を返す
+            return True if len(dfCheck) > 0 else False
+        elif searchOption == self.findOption.cacheAndDataBase:
+            
+            # セッションチェック
+            self.__checkSessionOpen(False, False)
+            # クエリ実行
+            q = self.filterExpr(w).all()
+            # セッションチェック
+            self.__checkSessionClose()
+            # 戻り値を返す
+            return True if len(dfCheck) > 0 or len(q) > 0 else False
+
     def existsPKeyRec(self, row):
         
         """
@@ -553,7 +756,7 @@ class recset(Generic[T]):
         """
 
         # セッションチェック
-        self.__checkSession(False, False)
+        self.__checkSessionOpen(False, False)
         
         # 主キーを条件として該当レコードが存在するか確認
         w = self.__getPKeyFilter_sa(row)
@@ -572,22 +775,6 @@ class recset(Generic[T]):
         # 結果を返す
         return len(q) > 0
 
-    def find(self, w):
-        
-        """
-        現在保持しているレコードセットに指定した条件に合致するレコードが存在するか返す
-
-        Parameters
-        ----------
-        w : Any
-            抽出条件
-        """
-        
-        # 条件抽出
-        dfCheck = self.__dfrows[w]
-
-        return True if len(dfCheck) > 0 else False
-
     def merge(self, srcRecset):
         
         """
@@ -641,33 +828,7 @@ class recset(Generic[T]):
         
         # fields再構築
         self.__fields = fields(self.__rows, self.__dfrows)
-        
-    def __checkSession(self, __autoCommit = True, __beginTransaction = False):
-
-        """
-        セッションチェック
-
-        Parameters
-        ----------
-        __autoCommit : bool
-            AutoCommit
-        __beginTransaction : bool
-            Sessionを開いた時にトランザクションを開始するか
-
-        """
-        
-        # sessionがNoneの場合はautocommit=Onで接続を開く
-        if self.__session is None:
-            self.__da.openSession(__autoCommit)
-            self.__session = self.__da.session
-            self.__da.beginTransaction()
-        else:
-            # autocommit=Onの場合はtrunsactionを開始する
-            if self.__da.autoCommit and \
-               self.session.transaction is None and \
-               __beginTransaction:
-                self.__da.beginTransaction()
-            
+    
     def upsert(self):
 
         """
@@ -682,7 +843,7 @@ class recset(Generic[T]):
         """
 
         return self.update(True)
-        
+    
     def update(self, upsert = False) -> en.resultRegister:
         
         """
@@ -697,7 +858,7 @@ class recset(Generic[T]):
         try:
             
             # セッションチェック
-            self.__checkSession(True, True)
+            self.__checkSessionOpen(True, True)
             
             # 新規行はaddする
             newRows = [row for row in self.__rows if row._recset__rowState == self.rowState.added]
